<header>
  <div class="container">
    <h1 class="logo">PiScan Portal</h1>
    <nav>
      <ul class="menu">
        <li><a href="{{ url_for('status') }}">Home</a></li>
        <li><a href="{{ url_for('status') }}">About</a></li>
        <li><a href="{{ url_for('status') }}">Contact</a></li>
      </ul>
    </nav>
  </div>
</header>
